// FetAddSrv.java
//  Dean Brock <brock@cs.unca.edu>

package edu.unca.cs.csci363.brock.fetaddsvc ;

import java.rmi.* ;
import java.rmi.server.* ;
import edu.unca.cs.csci363.brock.fetadd.* ;

public class FetAddSvc extends UnicastRemoteObject implements FetAdd {

  private static final int MAXSTRINGSIZE =  7 ;
  private static final int MAXVARIABLES  = 10 ;

  private int numVariables = 0 ;
  private int nextSlot     = 0 ;
  private String[] ServerName  = new String[MAXVARIABLES] ;
  private int[]    ServerValue = new int[MAXVARIABLES] ;

  public FetAddSvc() throws RemoteException {
    super() ;
  }

  public synchronized void create(String Variable)
      throws RemoteException, ImproperVariable, VariableAlreadyDefined {

    if (Variable.length() == 0)
      throw new ImproperVariable("Zero length variable") ;
    if (Variable.length() > MAXSTRINGSIZE)
      throw new ImproperVariable("Variable is too long") ;
    for (int i=0; i<Variable.length(); ++i)
      if (! Character.isLowerCase(Variable.charAt(i)))
        throw new ImproperVariable("Variable has non-lowercase characters") ;
    for (int i=0; i<numVariables; ++i)
      if (Variable.equals(ServerName[i]))
        throw new VariableAlreadyDefined("Variable already defined") ;

    // All checks have passed!
    if (numVariables < MAXVARIABLES)
      ++numVariables ;
    // If we run out of variables, just throw away the oldest

    ServerName[nextSlot]  = Variable ;
    ServerValue[nextSlot] = 0 ;
    nextSlot = (nextSlot+1) % MAXVARIABLES ;
  }

  public synchronized int     increment(String Variable, int Value)
      throws RemoteException, ImproperVariable, UndefinedVariable {

    if (Variable.length() == 0)
      throw new ImproperVariable("Zero length variable") ;
    if (Variable.length() > MAXSTRINGSIZE)
      throw new ImproperVariable("Variable is too long") ;
    for (int i=0; i<Variable.length(); ++i)
      if (! Character.isLowerCase(Variable.charAt(i)))
        throw new ImproperVariable("Variable has non-lowercase characters") ;
    
    int i ;
    boolean varfound = false ;
    for (i=0; i<numVariables && !varfound; ++i)
      varfound = Variable.equals(ServerName[i]) ;

    if (!varfound)
      throw new UndefinedVariable("Variable not defined") ;

    ServerValue[i] += Value ;

    return ServerValue[i] ;
  }
}
